@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -65,7 +65,7 @@ typedef struct _XTextProperty
   int
     format;
 
-  unsigned long
+  size_t
     nitems;
 } XTextProperty;
 
