@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -39,31 +39,33 @@ typedef struct _MagicInfo
     offset;
 
   MagickBooleanType
+    exempt,
     stealth;
 
   struct _MagicInfo
     *previous,
     *next;  /* deprecated, use GetMagicInfoList() */
 
-  unsigned long
+  size_t
     signature;
 } MagicInfo;
 
 extern MagickExport char
-  **GetMagicList(const char *,unsigned long *,ExceptionInfo *);
+  **GetMagicList(const char *,size_t *,ExceptionInfo *);
 
 extern MagickExport const char
   *GetMagicName(const MagicInfo *);
 
 extern MagickExport MagickBooleanType
-  ListMagicInfo(FILE *,ExceptionInfo *);
+  ListMagicInfo(FILE *,ExceptionInfo *),
+  MagicComponentGenesis(void);
 
 extern MagickExport const MagicInfo
   *GetMagicInfo(const unsigned char *,const size_t,ExceptionInfo *),
-  **GetMagicInfoList(const char *,unsigned long *,ExceptionInfo *);
+  **GetMagicInfoList(const char *,size_t *,ExceptionInfo *);
 
 extern MagickExport void
-  DestroyMagicList(void);
+  MagicComponentTerminus(void);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
