@@ -13,11 +13,11 @@
 %                   MagickCore Version and Copyright Methods                  %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2013 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,6 +46,7 @@
 #include "magick/string_.h"
 #include "magick/utility.h"
 #include "magick/version.h"
+#include "magick/version-private.h"
 
 /*
   Define declarations.
@@ -140,7 +141,7 @@ MagickExport const char *GetMagickDelegates(void)
 #if defined(MAGICKCORE_JPEG_DELEGATE) && defined(MAGICKCORE_PNG_DELEGATE)
   "jng "
 #endif
-#if defined(MAGICKCORE_JP2_DELEGATE)
+#if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   "jp2 "
 #endif
 #if defined(MAGICKCORE_JPEG_DELEGATE)
@@ -161,9 +162,6 @@ MagickExport const char *GetMagickDelegates(void)
 #if defined(MAGICKCORE_OPENEXR_DELEGATE)
   "openexr "
 #endif
-#if defined(MAGICKCORE_PANGO_DELEGATE)
-  "pango "
-#endif
 #if defined(MAGICKCORE_PANGOCAIRO_DELEGATE)
   "pangocairo "
 #endif
@@ -173,9 +171,6 @@ MagickExport const char *GetMagickDelegates(void)
 #if defined(MAGICKCORE_DPS_DELEGATE) || defined(MAGICKCORE_GS_DELEGATE) || defined(WIN32)
   "ps "
 #endif
-#if defined(MAGICKCORE_PNG_DELEGATE)
-  "png "
-#endif
 #if defined(MAGICKCORE_RSVG_DELEGATE)
   "rsvg "
 #endif
@@ -226,11 +221,17 @@ MagickExport const char *GetMagickFeatures(void)
 #if defined(MAGICKCORE_HDRI_SUPPORT)
   " HDRI"
 #endif
+#if defined(MAGICKCORE_BUILD_MODULES) || defined(_DLL)
+  " Modules"
+#endif
 #if defined(MAGICKCORE_OPENCL_SUPPORT)
   " OpenCL"
 #endif
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   " OpenMP"
+#endif
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_DEBUG)
+  " Debug"
 #endif
   ;
 }
@@ -553,5 +554,6 @@ MagickExport void ListMagickVersion(FILE *file)
     GetMagickVersion((size_t *) NULL));
   (void) FormatLocaleFile(file,"Copyright: %s\n",GetMagickCopyright());
   (void) FormatLocaleFile(file,"Features: %s\n",GetMagickFeatures());
-  (void) FormatLocaleFile(file,"Delegates: %s\n\n",GetMagickDelegates());
+  (void) FormatLocaleFile(file,"Delegates (built-in): %s\n\n",
+    GetMagickDelegates());
 }
